# Kanji Cards
# by Pter Salvi
# using the database of Szabolcs Varga

# Az ablak felptse

wm title . "Kanji Cards"
wm minsize . 200 250
wm maxsize . 200 250

font create sajat_font -size 8 -family helvetica

button .b_uj -text "Open Kanji list..." -command { nyitas } -font sajat_font
button .b_kilep -text "Exit" -command exit -font sajat_font
canvas .c_kanji -width 98 -height 98
button .b_kov -text "Next Kanji" -command { kovetkezo } -font sajat_font
button .b_mutat -text "Show Kanji" -command { megmutat } -font sajat_font
button .b_on -text "On-yomi" -command { on_yomi 0 } -font sajat_font
button .b_kun -text "Kun-yomi" -command { kun_yomi 0 } -font sajat_font
label .l_on -justify left -wraplength 96 -textvariable onyomi -font sajat_font
label .l_kun -justify left -wraplength 96 -textvariable kunyomi -font sajat_font
label .l_def -wraplength 196 -textvariable definicio -font sajat_font

place .b_uj    -width 96  -height 25 -x 2   -y 2
place .c_kanji -width 98  -height 98 -x 1   -y 28
place .b_on    -width 96  -height 39 -x 2   -y 127
place .b_kun   -width 96  -height 39 -x 2   -y 168
place .l_def   -width 196 -height 39 -x 2   -y 209
place .b_kilep -width 96  -height 25 -x 102 -y 2
place .b_kov   -width 96  -height 25 -x 102 -y 44
place .b_mutat -width 96  -height 25 -x 102 -y 85
place .l_on    -width 96  -height 39 -x 102 -y 127
place .l_kun   -width 96  -height 39 -x 102 -y 168

# Pontkirajzols
proc pont {x y szin} {
    eval .c_kanji create rectangle [expr $x * 3 - 2 + 12] [expr $y * 3 - 2 + 12] \
	    [expr $x * 3 + 12] [expr $y * 3 + 12] -fill $szin
}

# Hibazenet
proc hiba {uzenet} {
    wm withdraw .
    toplevel .warning -width 250 -height 50 
    wm minsize .warning 250 50
    wm maxsize .warning 250 50
    wm title .warning "Error!"
    label .warning.msg -text $uzenet
    button .warning.ok -text "Exit" -command exit
    focus -force .warning 
    pack .warning.msg .warning.ok -side top -fill x -pady 2 -padx 2 
}

# Kanji kikdolsa
proc decode {c} {
    binary scan $c c d
    for {set i 7} {$i >= 0} {incr i -1} {
	if {[expr $d & (1 << $i)]} {
	    append result "O"
	} else {
	    append result "."
	}
    }
    return $result
}

# Kanji kpnek kiolvassa
proc extract {szam} {
    if {![file exists "kanji.pat"]} {
	hiba "Cannot open the file kanji.pat!"
    } else {
	set f [open "kanji.pat" r]
	fconfigure $f -translation binary
	seek $f [expr ($szam - 1) * 72]
	for {set i 0} {$i < 24} {incr i 1} {
	    for {set j 0} {$j < 3} {incr j 1} {
		set c [read $f 1]
		append kep [decode $c] 
	    }
	    append kep "\n"
	}
	close $f
    }
    return $kep
}

# Kanjikirajzols kpfjlbl
proc rajzol {szam} {
    set kep [extract $szam] 
    for {set j 1} {$j <= 24} {incr j 1} {
	for {set i 1} {$i <= 24} {incr i 1} {
	    if {[string compare [string index $kep 0] 0] == 1} {
		pont $i $j black
	    } 
	    set kep [string range $kep 1 end]
	}
	set kep [string range $kep 1 end]	
    }
}

proc megmutat {} {
    global definicio
    global defstring
    on_yomi 1
    kun_yomi 1
    set definicio $defstring
}

# Kvetkez kanji elksztse
proc kovetkezo {} {
    global k_id
    global onyomi
    global kunyomi
    global definicio
    global defstring
    global fname
    global kov_index
    global sorrend

    if {![file exists $fname]} {
	hiba "Cannot open the file $fname!"
    } else {
	set f [open $fname r]
	set sorszam [gets $f]
	incr kov_index 1
	if {$kov_index > $sorszam} {
	    kever
	    set kov_index 1
	}
	for {set i 0} {$i < $sorrend($kov_index)} {incr i 1} { set k_id [gets $f] }
	close $f
    }

    set onyomi ""
    set kunyomi ""
    set definicio ""
    eval .c_kanji create rectangle 1 1 96 96 -fill white
    if {![file exists "kanji.hun"]} {
	hiba "Cannot open the file kanji.hun!"
    } else {
	set f [open "kanji.hun" r]
	fconfigure $f -encoding iso8859-1
	for {set i 0} {$i < [expr $k_id * 3]} {incr i 1} {
	    gets $f sor
	}
	close $f
	set defstring $sor
	rajzol $k_id
    }
}

# On-yomi megmutatsa / eltntetse
proc on_yomi {muszaj} {
    global onyomi
    global k_id
    if {[string compare $onyomi ""] == 0 || $muszaj} {
	if {![file exists "kanji.hun"]} {
	    hiba "Cannot open the file kanji.hun!"
	} else {
	    set f [open "kanji.hun" r]
	    for {set i 0} {$i < [expr $k_id * 3 - 2]} {incr i 1} {
		gets $f sor
	    }
	    close $f
	    set onyomi $sor
	}
    } else {
	set onyomi "" 
    }
}

# Kun-yomi megmutatsa / eltntetse
proc kun_yomi {muszaj} {
    global kunyomi
    global k_id
    if {[string compare $kunyomi ""] == 0 || $muszaj} {
	if {![file exists "kanji.hun"]} {
	    hiba "Cannot open the file kanji.hun!"
	} else {
	    set f [open "kanji.hun" r]
	    for {set i 0} {$i < [expr $k_id * 3 - 1]} {incr i 1} {
		gets $f sor
	    }
	    close $f
	    set kunyomi $sor
	}
    } else {
	set kunyomi "";
    }
}

# Kanji sorrend keverse
proc kever {} {
    global kov_index
    global fname
    global sorrend

    if {![file exists $fname]} {
	hiba "Cannot open the file $fname!"
    } else {
	set f [open $fname r]
	set sorszam [gets $f]
	close $f
    }

    for {set i 1} {$i <= $sorszam} {incr i 1} {
	set sorrend($i) $i
    }

    for {set i 1} {$i < $sorszam} {incr i 1} {
	set veletlen [expr int(rand() * ($sorszam - $i)) + $i + 1]
	set temp $sorrend($i)
	set sorrend($i) $sorrend($veletlen)
	set sorrend($veletlen) $temp
    }
}

# Kanji-lista vltoztats
proc nyitas {} {
    global fname
    global kov_index
    set types {
	{ {Kanji-list files} {.lst} }
	{ {All files} * }
    }
    set fn [tk_getOpenFile -filetypes $types -defaultextension lst -parent . \
	    -title "Select Kanji list"]
    if { $fn != "" } { 
	set fname $fn 
	kever
	set kov_index 0
	kovetkezo
    }
}

# Els kanji betltse
set fname "kanji.lst"
kever
set kov_index 0
kovetkezo
